@@ -2,28 +2,28 @@
 
 """Class for performing HTTP client requests via libcurl"""
 
+import json
+import contextlib
+
 import nilmdb.utils
 import nilmdb.client.httpclient
 from nilmdb.client.errors import ClientError
-
-import time
-import simplejson as json
-import contextlib
-
 from nilmdb.utils.time import timestamp_to_string, string_to_timestamp
 
+
 def extract_timestamp(line):
     """Extract just the timestamp from a line of data text"""
     return string_to_timestamp(line.split()[0])
 
-class Client(object):
+
+class Client():
     """Main client interface to the Nilm database."""
 
-    def __init__(self, url, post_json = False, keep_alive = False):
+    def __init__(self, url, post_json=False):
         """Initialize client with given URL.  If post_json is true,
         POST requests are sent with Content-Type 'application/json'
         instead of the default 'x-www-form-urlencoded'."""
-        self.http = nilmdb.client.httpclient.HTTPClient(url, post_json, keep_alive = keep_alive)
+        self.http = nilmdb.client.httpclient.HTTPClient(url, post_json)
         self.post_json = post_json
 
     # __enter__/__exit__ allow this class to be a context manager
@@ -38,7 +38,7 @@ class Client(object):
         if self.post_json:
             # If we're posting as JSON, we don't need to encode it further here
             return data
-        return json.dumps(data, separators=(',',':'))
+        return json.dumps(data, separators=(',', ':'))
 
     def close(self):
         """Close the connection; safe to call multiple times"""
@@ -57,7 +57,7 @@ class Client(object):
         as a dictionary."""
         return self.http.get("dbinfo")
 
-    def stream_list(self, path = None, layout = None, extended = False):
+    def stream_list(self, path=None, layout=None, extended=False):
         """Return a sorted list of [path, layout] lists.  If 'path' or
         'layout' are specified, only return streams that match those
         exact values.  If 'extended' is True, the returned lists have
@@ -71,11 +71,11 @@ class Client(object):
         if extended:
             params["extended"] = 1
         streams = self.http.get("stream/list", params)
-        return nilmdb.utils.sort.sort_human(streams, key = lambda s: s[0])
+        return nilmdb.utils.sort.sort_human(streams, key=lambda s: s[0])
 
-    def stream_get_metadata(self, path, keys = None):
+    def stream_get_metadata(self, path, keys=None):
         """Get stream metadata"""
-        params = { "path": path }
+        params = {"path": path}
         if keys is not None:
             params["key"] = keys
         return self.http.get("stream/get_metadata", params)
@@ -86,7 +86,7 @@ class Client(object):
         params = {
             "path": path,
             "data": self._json_post_param(data)
-            }
+        }
         return self.http.post("stream/set_metadata", params)
 
     def stream_update_metadata(self, path, data):
@@ -94,27 +94,33 @@ class Client(object):
         params = {
             "path": path,
             "data": self._json_post_param(data)
-            }
+        }
         return self.http.post("stream/update_metadata", params)
 
     def stream_create(self, path, layout):
         """Create a new stream"""
-        params = { "path": path,
-                   "layout" : layout }
+        params = {
+            "path": path,
+            "layout": layout
+        }
         return self.http.post("stream/create", params)
 
     def stream_destroy(self, path):
         """Delete stream.  Fails if any data is still present."""
-        params = { "path": path }
+        params = {
+            "path": path
+        }
         return self.http.post("stream/destroy", params)
 
     def stream_rename(self, oldpath, newpath):
         """Rename a stream."""
-        params = { "oldpath": oldpath,
-                   "newpath": newpath }
+        params = {
+            "oldpath": oldpath,
+            "newpath": newpath
+        }
         return self.http.post("stream/rename", params)
 
-    def stream_remove(self, path, start = None, end = None):
+    def stream_remove(self, path, start=None, end=None):
         """Remove data from the specified time range"""
         params = {
             "path": path
@@ -129,7 +135,7 @@ class Client(object):
         return total
 
     @contextlib.contextmanager
-    def stream_insert_context(self, path, start = None, end = None):
+    def stream_insert_context(self, path, start=None, end=None):
         """Return a context manager that allows data to be efficiently
         inserted into a stream in a piecewise manner.  Data is
         provided as ASCII lines, and is aggregated and sent to the
@@ -152,21 +158,21 @@ class Client(object):
         ctx.finalize()
         ctx.destroy()
 
-    def stream_insert(self, path, data, start = None, end = None):
+    def stream_insert(self, path, data, start=None, end=None):
         """Insert rows of data into a stream.  data should be a string
         or iterable that provides ASCII data that matches the database
         layout for path.  Data is passed through stream_insert_context,
         so it will be broken into reasonably-sized chunks and
         start/end will be deduced if missing."""
         with self.stream_insert_context(path, start, end) as ctx:
-            if isinstance(data, basestring):
+            if isinstance(data, bytes):
                 ctx.insert(data)
             else:
                 for chunk in data:
                     ctx.insert(chunk)
         return ctx.last_response
 
-    def stream_insert_block(self, path, data, start, end, binary = False):
+    def stream_insert_block(self, path, data, start, end, binary=False):
         """Insert a single fixed block of data into the stream.  It is
         sent directly to the server in one block with no further
         processing.
@@ -181,9 +187,9 @@ class Client(object):
         }
         if binary:
             params["binary"] = 1
-        return self.http.put("stream/insert", data, params, binary = binary)
+        return self.http.put("stream/insert", data, params)
 
-    def stream_intervals(self, path, start = None, end = None, diffpath = None):
+    def stream_intervals(self, path, start=None, end=None, diffpath=None):
         """
         Return a generator that yields each stream interval.
 
@@ -201,8 +207,8 @@ class Client(object):
             params["end"] = timestamp_to_string(end)
         return self.http.get_gen("stream/intervals", params)
 
-    def stream_extract(self, path, start = None, end = None,
-                       count = False, markup = False, binary = False):
+    def stream_extract(self, path, start=None, end=None,
+                       count=False, markup=False, binary=False):
         """
         Extract data from a stream.  Returns a generator that yields
         lines of ASCII-formatted data that matches the database
@@ -232,17 +238,18 @@ class Client(object):
             params["markup"] = 1
         if binary:
             params["binary"] = 1
-        return self.http.get_gen("stream/extract", params, binary = binary)
+        return self.http.get_gen("stream/extract", params, binary=binary)
 
-    def stream_count(self, path, start = None, end = None):
+    def stream_count(self, path, start=None, end=None):
         """
         Return the number of rows of data in the stream that satisfy
         the given timestamps.
         """
-        counts = list(self.stream_extract(path, start, end, count = True))
+        counts = list(self.stream_extract(path, start, end, count=True))
         return int(counts[0])
 
-class StreamInserter(object):
+
+class StreamInserter():
     """Object returned by stream_insert_context() that manages
     the insertion of rows of data into a particular path.
 
@@ -330,8 +337,8 @@ class StreamInserter(object):
 
         # Send the block once we have enough data
         if self._block_len >= maxdata:
-            self._send_block(final = False)
-            if self._block_len >= self._max_data_after_send: # pragma: no cover
+            self._send_block(final=False)
+            if self._block_len >= self._max_data_after_send:
                 raise ValueError("too much data left over after trying"
                                  " to send intermediate block; is it"
                                  " missing newlines or malformed?")
@@ -357,12 +364,12 @@ class StreamInserter(object):
 
         If more data is inserted after a finalize(), it will become
         part of a new interval and there may be a gap left in-between."""
-        self._send_block(final = True)
+        self._send_block(final=True)
 
     def send(self):
         """Send any data that we might have buffered up.  Does not affect
         any other treatment of timestamps or endpoints."""
-        self._send_block(final = False)
+        self._send_block(final=False)
 
     def _get_first_noncomment(self, block):
         """Return the (start, end) indices of the first full line in
@@ -370,10 +377,10 @@ class StreamInserter(object):
         there isn't one."""
         start = 0
         while True:
-            end = block.find('\n', start)
+            end = block.find(b'\n', start)
             if end < 0:
                 raise IndexError
-            if block[start] != '#':
+            if block[start] != b'#'[0]:
                 return (start, (end + 1))
             start = end + 1
 
@@ -381,22 +388,22 @@ class StreamInserter(object):
         """Return the (start, end) indices of the last full line in
         block[:length] that isn't a comment, or raise IndexError if
         there isn't one."""
-        end = block.rfind('\n')
+        end = block.rfind(b'\n')
         if end <= 0:
             raise IndexError
         while True:
-            start = block.rfind('\n', 0, end)
-            if block[start + 1] != '#':
+            start = block.rfind(b'\n', 0, end)
+            if block[start + 1] != b'#'[0]:
                 return ((start + 1), end)
             if start == -1:
                 raise IndexError
             end = start
 
-    def _send_block(self, final = False):
+    def _send_block(self, final=False):
         """Send data currently in the block.  The data sent will
         consist of full lines only, so some might be left over."""
         # Build the full string to send
-        block = "".join(self._block_data)
+        block = b"".join(self._block_data)
 
         start_ts = self._interval_start
         if start_ts is None:
@@ -405,7 +412,7 @@ class StreamInserter(object):
                 (spos, epos) = self._get_first_noncomment(block)
                 start_ts = extract_timestamp(block[spos:epos])
             except (ValueError, IndexError):
-                pass # no timestamp is OK, if we have no data
+                pass  # no timestamp is OK, if we have no data
 
         if final:
             # For a final block, it must end in a newline, and the
@@ -413,14 +420,14 @@ class StreamInserter(object):
             # or the timestamp of the last line plus epsilon.
             end_ts = self._interval_end
             try:
-                if block[-1] != '\n':
+                if block[-1] != b'\n'[0]:
                     raise ValueError("final block didn't end with a newline")
                 if end_ts is None:
                     (spos, epos) = self._get_last_noncomment(block)
                     end_ts = extract_timestamp(block[spos:epos])
                     end_ts += nilmdb.utils.time.epsilon
             except (ValueError, IndexError):
-                pass # no timestamp is OK, if we have no data
+                pass  # no timestamp is OK, if we have no data
             self._block_data = []
             self._block_len = 0
 
@@ -447,7 +454,7 @@ class StreamInserter(object):
                 # the server complain so that the error is the same
                 # as if we hadn't done this chunking.
                 end_ts = self._interval_end
-            self._block_data = [ block[spos:] ]
+            self._block_data = [block[spos:]]
             self._block_len = (epos - spos)
             block = block[:spos]
 
@@ -465,6 +472,6 @@ class StreamInserter(object):
 
         # Send it
         self.last_response = self._client.stream_insert_block(
-            self._path, block, start_ts, end_ts, binary = False)
+            self._path, block, start_ts, end_ts, binary=False)
 
         return
