@@ -1,5 +1,7 @@
 # -*- coding: utf-8 -*-
 
+raise Exception("todo: fix path bytes issues")
+
 """Check database consistency, with some ability to fix problems.
 This should be able to fix cases where a database gets corrupted due
 to unexpected system shutdown, and detect other cases that may cause
@@ -10,7 +12,7 @@ import nilmdb.server
 import nilmdb.client.numpyclient
 from nilmdb.utils.interval import IntervalError
 from nilmdb.server.interval import Interval, IntervalSet
-from nilmdb.utils.printf import *
+from nilmdb.utils.printf import printf, fprintf, sprintf
 from nilmdb.utils.time import timestamp_to_string
 
 from collections import defaultdict
@@ -21,7 +23,7 @@ import progressbar
 import re
 import time
 import shutil
-import cPickle as pickle
+import pickle
 import numpy
 
 class FsckError(Exception):
@@ -179,7 +181,7 @@ class Fsck(object):
     ### Check streams and basic interval overlap
 
     def check_streams(self):
-        ids = self.stream_path.keys()
+        ids = list(self.stream_path.keys())
         log("checking %s streams\n", "{:,d}".format(len(ids)))
         with Progress(len(ids)) as pbar:
             for i, sid in enumerate(ids):
@@ -187,7 +189,7 @@ class Fsck(object):
                 path = self.stream_path[sid]
 
                 # unique path, valid layout
-                if self.stream_path.values().count(path) != 1:
+                if list(self.stream_path.values()).count(path) != 1:
                     raise FsckError("duplicated path %s", path)
                 layout = self.stream_layout[sid].split('_')[0]
                 if layout not in ('int8', 'int16', 'int32', 'int64',
@@ -269,7 +271,7 @@ class Fsck(object):
         for subdir in subdirs:
             # Find all files in that dir
             subpath = os.path.join(bulk, subdir)
-            files = filter(regex.search, os.listdir(subpath))
+            files = list(filter(regex.search, os.listdir(subpath)))
             if not files:
                 self.fix_empty_subdir(subpath)
                 raise RetryFsck
@@ -315,7 +317,7 @@ class Fsck(object):
     ### Check interval endpoints
 
     def check_intervals(self):
-        total_ints = sum(len(x) for x in self.stream_interval.values())
+        total_ints = sum(len(x) for x in list(self.stream_interval.values()))
         log("checking %s intervals\n", "{:,d}".format(total_ints))
         done = 0
         with Progress(total_ints) as pbar:
@@ -398,7 +400,7 @@ class Fsck(object):
 
     def check_data(self):
         total_rows = sum(sum((y[3] - y[2]) for y in x)
-                         for x in self.stream_interval.values())
+                         for x in list(self.stream_interval.values()))
         log("checking %s rows of data\n", "{:,d}".format(total_rows))
         done = 0
         with Progress(total_rows) as pbar:
