@@ -3,7 +3,7 @@ from nilmdb.utils.printf import *
 
 from nose.tools import *
 from nose.tools import assert_raises
-from cStringIO import StringIO
+from io import StringIO
 import sys
 
 from testutil.helpers import *
